package okay.database;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import okay.OkayRuntimeException;

/**
 * Manipulates with database.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class Database {

    private static final String SQLITEEXTENSION = ".sqlite";

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private Database() {
        //Not meant to be instantiated.        
    }

    /**
     * Creates new database with the given name.
     *
     * @param databaseName
     */
    public static void createDatabase(String databaseName) {
        if (existsDatabase(databaseName)) {
            throw new OkayRuntimeException("A database with the given name already exists.");
        }
        Connection connection = null;
        try {
            Class.forName("org.sqlite.JDBC");
            connection = DriverManager.getConnection("jdbc:sqlite:" + convertDatabaseNameToFileName(databaseName));
        } catch (Exception e) {
            Logger.getLogger(Database.class.getName()).log(Level.SEVERE, null, e);
            throw new OkayRuntimeException("New database was not created.");
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException ex) {
                    Logger.getLogger(Database.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * Deletes forever the database with the given name.
     *
     * @param databaseName
     */
    public static void dropDatabase(String databaseName) {
        if (!existsDatabase(databaseName)) {
            throw new OkayRuntimeException("There is no database with the given name.");
        }
        File file = new File("./" + convertDatabaseNameToFileName(databaseName));
        boolean delete = file.delete();
        if (!delete) {
            throw new OkayRuntimeException("The database was not dropped.");
        }
    }

    /**
     * @param databaseName
     * @return true, if a database with the name exists, otherwise false.
     */
    public static boolean existsDatabase(String databaseName) {
        File file = new File("./" + convertDatabaseNameToFileName(databaseName));
        return (file.exists()) && !(file.isDirectory());
    }

    /**
     * Creates a database connection for the database with the given name.
     *
     * @param databaseName
     * @return database connection to the database with the given name
     */
    public static DatabaseConnection createDatabaseConnection(String databaseName) {
        if ((databaseName != "") && (!existsDatabase(databaseName))) {
            throw new OkayRuntimeException("There is no database with the given name.");
        }
        return new DatabaseConnection(databaseName);
    }

    /**
     * Creates a new in memory database and return its connection
     *
     * @return database connection
     */
    public static DatabaseConnection createInMemoryDatabaseConnection() {
        return createDatabaseConnection("");
    }

    private static String convertDatabaseNameToFileName(String databaseName) {
        return databaseName + SQLITEEXTENSION;
    }

}
