package okay.datetime;

import okay.OkayRuntimeException;

/**
 * Represents Date without time zone information.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public final class LocalDate {

    private final int year;
    private final int month;
    private final int day;

    /**
     * Constructor
     *
     * @param year Year of this date.
     * @param month Month of this date.
     * @param day Day of this date.
     * @exception OkayRuntimeException if parameters are invalid or have wrong
     * combination.
     */
    public LocalDate(int year, int month, int day) {
        LocalDate.checkInputValuesForDateAndIfThereIsAnInvalidOneThrowException(year, month, day);
        this.year = year;
        this.month = month;
        this.day = day;
    }

    /**
     * Checks input data for date.
     *
     * @param year
     * @param month
     * @param day
     * @exception OkayRuntimeException if input data for date are invalid or
     * have wrong combination.<br />
     * This will throw OkayRuntimeException:
     * {@code new LocalDateTime(2015,2,29)}<br />
     * This will throw OkayRuntimeException:
     * {@code new LocalDateTime(2013,6,31)}<br />
     * This will not throw OkayRuntimeException:
     * {@code new LocalDateTime(2013,6,30)}<br />
     */
    private static void checkInputValuesForDateAndIfThereIsAnInvalidOneThrowException(int year, int month, int day) {
        if (!DateUnitsValidator.isMonthValid(month)) {
            throw new OkayRuntimeException("Month is not valid.");
        }
        if (!DateUnitsValidator.isDayValid(day)) {
            throw new OkayRuntimeException("Day is not valid.");
        }
        if (!DateUnitsValidator.hasDateValidCombination(year, month, day)) {
            throw new OkayRuntimeException("Date has not valid combination.");
            
        }
    }

    /**
     * Checks if year is leap. A year is leap if this year is divided by 4 and
     * there remainder 0. In other words this year is divisible by 4.
     *
     * @param year
     * @return true if the year is leap, otherwise false.
     */
    public static boolean isYearLeap(int year) {
        return (year % 4) == 0;
    }

    /**
     *
     * @return year of this date.
     */
    public int getYear() {
        return this.year;
    }

    /**
     *
     * @return month of this date.
     */
    public int getMonth() {
        return this.month;
    }

    /**
     *
     * @return day of this date.
     */
    public int getDay() {
        return this.day;
    }

    @Override
    public String toString() {
        StringBuilder stringBuilder;
        stringBuilder = new StringBuilder();
        stringBuilder.append(this.getYear());
        stringBuilder.append("-");
        if (month < 10) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getMonth());
        stringBuilder.append("-");
        if (day < 10) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getDay());
        return stringBuilder.toString();
    }
}
